@@ -40,6 +40,21 @@ safe_module_require_class('fs', 'lcr_module');
 safe_module_require_class('sng', 'rest_module');
 safe_module_require_class('sng', 'update_module');
 safe_module_require_class('sng', 'network_service');
+class Nsc_snort_update_configuration_class extends Safe_configuration_user_class
+{
+    public function __construct($port_list)
+    {
+        parent::__construct('Intrusion Detection', Safe_configuration_class::CFG_UPDATE, $port_list);
+    }
+    public function run($node)
+    {
+        return $node->execute('sudo ' . $node->configuration_manager()->directory('binary') . '/snort-update-config --ports "' . implode(',', $this->data()) . '"');
+    }
+    public function file_name()
+    {
+        return 'Intrusion Detection - Port List = ' . implode(',', $this->data());
+    }
+}
 class Nsc_application_class extends Fs_application_class
 {
     const NSC_SERVICE_CONF_DIR = '/etc/sysconfig/';
@@ -56,16 +71,15 @@ class Nsc_application_class extends Fs_application_class
      * @return 
      */
     public function register_modules(){
+        // register base modules
+        parent::register_modules();
         // Register Freeswitch modules
         $this->register_module(new Fs_sngms_module_class($this));
         $this->register_module(new Fs_cdr_module_class($this));
         $this->register_module(new Fs_enum_module_class($this));
         $this->register_module(new Fs_radius_module_class($this));
-        $this->register_module(new Fs_dialplan_module_class($this));
         $this->register_module(new Fs_cac_module_class($this));
         $this->register_module(new Fs_lcr_module_class($this));
-        // register base modules
-        parent::register_modules();
         // Overloaded NSC modules
         $this->register_module(new Nsc_directory_module_class($this));
         $this->register_module(new Nsc_media_module_class($this));
@@ -88,6 +102,7 @@ class Nsc_application_class extends Fs_application_class
             'INSTALLDIR=' . $config_manager->directory('base') ,
             'CUSTOM_PYTHON_PATH=' . $config_manager->directory('python-packages')
         );
+        
         // Get all the services configuration
         $services = $this->node()->software()->services();
         foreach ($services as $service) {
@@ -117,7 +132,34 @@ class Nsc_application_class extends Fs_application_class
     {
         // Services configuration
         if (!$this->_generate_services_configuration($config_manager)) return false;
+        // Update snort
+        $this->update_intrusion_detection($config_manager);
         return parent::generate_config($config_manager);
     }
+    private function update_intrusion_detection($config_manager)
+    {
+        $port_list = array();
+        $sip_profiles = $this->sip_profiles();
+        foreach ($sip_profiles as $sip) {
+            $sip->get_sip_transport_info($trans, $port);
+            if ($trans != 'tls') {
+                if (FALSE == in_array($port, $port_list)) $port_list[] = $port;
+            }
+        }
+        $config_manager->add_config(new Nsc_snort_update_configuration_class($port_list));
+    }
+    /**
+     * @brief Reload generate/config handler
+     *
+     * @param[in out] $config_manager
+     * @param[in out] $obj_type
+     *
+     * @return
+     */
+    public function reload_generate_config(&$config_manager, $obj_type=null)
+    {
+        if (!$this->_generate_services_configuration($config_manager)) return false;
+        return parent::reload_generate_config($config_manager, $obj_type);
+    }
 }
 ?>
