@@ -102,20 +102,16 @@ class Nsc_domain_class extends Fs_domain_class
      * Checks to see if object can be deleted
      * @see Safe_serializable_object_class::can_dispose()
      */
-    public function can_dispose()
+    public function can_dispose(&$reason)
     {
-        //Check if media profile used in any sip profiles
-        $this->_the_app = $this->_node->software()->application();
-        $this->_sip_profiles = $this->_the_app->sip_profiles();
-        $return = "";
-        foreach ($this->_sip_profiles as $profile) {
-            if (isset($profile)) {
-                if ($profile->domain_check($this->name())) {
-                    return false;
-                }
-            }
+        //Check if media profile used 
+        $usages = $this->node()->software()->application()->check_object_usage('directory','domain', $this->name());
+        if($usages){
+            $reason['msg'] = "  {$this->name()} is used by ";
+            $reason['obj'] = $usages;
+            return false;
         }
-        return parent::can_dispose();
+        return parent::can_dispose($reason);
     }
     public function is_sip_profile_used($sip_profile)
     {
@@ -128,5 +124,36 @@ class Nsc_domain_class extends Fs_domain_class
         $forward = $this->get_data_value('forward-registration', false);
         return ('true' == $forward);
     }
+    
+    public function api_create_user($name, $data=null) {
+        $user = new Nsc_domain_users_class($this->_node, $this->object_name() . '/user', $name);
+        $user->configure();
+        if($data) {
+            $user->set_data_values($data);
+        }
+        return $user;
+    }
+    public function summary($type = 'horizontal' , $long = false){
+        $table_line = array("Domain Name" => $this->name(),
+                "Forward" => $this->get_data_value('forward-registration'),
+                "Registrar Server" => ($this->is_forward_enabled()) ? $this->get_data_value('registrar-server') : 'N/A',
+                "Registrar Port" => ($this->is_forward_enabled()) ? $this->get_data_value('registrar-port') : 'N/A');
+        return array('data' => $table_line);
+    }
+    public function new_fields(&$config_manager, $type)
+    {
+        parent::new_fields($config_manager, $type);
+        $condition_number = count($this->get_aggregate_objects($type))+1;
+        $new_name = ucfirst($type) . $condition_number;
+        $config_manager->add_field('profile-name', 'Domain User ID', 'string', $new_name, 50);
+    }
+    public function sip_profile_check($sip_profile){
+        $prof_data = $this->get_data_values(false);
+        if($prof_data['forward-registration']){
+            return ($prof_data['registrar-profile'] == $sip_profile) ? true : false;
+        }else{
+            return false;
+        }
+    }
 }
 ?>
