@@ -32,12 +32,14 @@ class Nsc_sipsecmon_service_class extends Safe_service_class
     {
         parent::__construct($software, 'sipsecmon');
         $this->_node = $node;
+        // Override run_dir for ipd file
+        $this->_run_dir = '/usr/local/nsc/run';
     }
     //Information pulled for the object
     public function configure()
     {
         // Create the object
-        $cfg = new Nsc_sipsecmon_config_class($this->_node, parent::path() , $this->name());
+        $cfg = new Nsc_sipsecmon_config_class($this->_node, $this->object_name(), $this->name());
         // Create the fields
         $cfg->configure();
         // Synchronize with persistent DB (safe_object table)
@@ -52,6 +54,28 @@ class Nsc_sipsecmon_service_class extends Safe_service_class
                 'name' => 'Rules',
                 'dynamic' => true,
                 'base_path' => parent::object_name().'/rules',
+                'methods' => array(
+                           'create' => array(
+                                   'name' => 'Create',
+                                   'description' => 'Create a rule',
+                                   'request' => 'POST',
+                           ),
+                           'retrieve' => array(
+                                   'name' => 'Retrieve',
+                                   'description' => 'Retrieve a rule',
+                                   'request' => 'GET',
+                           ),
+                           'update' => array(
+                                   'name' => 'Update',
+                                   'description' => 'Update a rule',
+                                   'request' => 'POST',
+                           ),
+                           'delete' => array(
+                                   'name' => 'Delete',
+                                   'description' => 'Delete a rule',
+                                   'request' => 'POST',
+                           ),
+                   ),
             )   
         );  
         return parent::configure();
@@ -84,8 +108,9 @@ class Nsc_sipsecmon_service_class extends Safe_service_class
     * @return 
     */
     public function create_aggregate_object($type,$name){
-        return new Nsc_sipsecmon_rules_config_class(
-            $this->_node, parent::object_name() . '/rules', $name);
+         $obj = new Nsc_sipsecmon_rules_config_class($this->_node, parent::object_name() . '/rules', $name);
+         $obj->configure();
+         return $obj;
     }
     /**
      * Returns sip secmon rules in an array
@@ -250,6 +275,13 @@ class Nsc_sipsecmon_service_class extends Safe_service_class
                     'download',
                     'view'
                 ) ,
+                'methods' => array(
+                        'download' => array(
+                                'name' => 'Download',
+                                'description' => array( 'Download log files.' ),
+                                'request' => 'GET',
+                        )
+                )
             )
         );
         return $data;
@@ -265,7 +297,7 @@ class Nsc_sipsecmon_service_class extends Safe_service_class
     }   
     public function reload_clear_modified($obj_type=null)
     {
-        $this->clear_configuration_modified();
+        return $this->clear_configuration_modified();
     }
     public function post_write_config()
     {
@@ -291,4 +323,82 @@ class Nsc_sipsecmon_service_class extends Safe_service_class
             break;
         }
     }
+    
+    /*
+    public function rules()
+    {
+        return $this->get_aggregate_objects('rule');
+    }*/
+    /**
+     * @brief
+     *
+     * @param[in out] $name
+     * @param[in out] $data
+     *
+     * @return
+     */
+    public function api_create_rules($name, $data=null) {
+        $obj = new Nsc_sipsecmon_rules_config_class($this->_node, parent::object_name() . '/rules', $name);
+        $obj->configure();
+        if($data) {
+            $obj->set_data_values($data);
+        }
+        return $obj;
+    }
+    /**
+     * @brief
+     *
+     * @param[in out] $name
+     * @param[in out] $data
+     *
+     * @return
+     */
+    public function api_retrieve_rules($name, $data=null) {
+        $rules = $this->rules();
+        return $rules[$name];
+    }
+    /**
+     * @brief
+     *
+     * @param[in out] $name
+     * @param[in out] $data
+     *
+     * @return
+     */
+    public function api_update_rules($name, $data=null, &$output = null) {
+        $rule = $this->api_retrieve_rules($name);
+        if($rule) {
+            if ($rule->validate($data,$output)) {
+                if (true == $rule->save()) {
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
+    /**
+     * @brief
+     *
+     * @param[in out] $name
+     * @param[in out] $data
+     *
+     * @return
+     */
+    public function api_delete_rules($name, $data=null, &$output = null) {
+        $rules = $this->rules();
+        if($rules[$name]) {
+            if($rules[$name]->can_dispose($output)) {
+                return $rules[$name]->dispose();
+            }
+        }
+        return false;
+    }
+    public function api_download_log($name=null, $data = null, &$output = null){
+        $this->download_data('log', $name);
+        exit();
+    }
+    public function can_restore($info, &$reason)
+    {
+        return true;
+    }
 }
