@@ -35,6 +35,11 @@ define('SAFE_CMD_HEAD', '/usr/bin/head');
 define('SAFE_CMD_REPLACE', '/usr/sbin/app-rename');
 define('SAFE_CMD_STDERR_REDIRECT', ' 2>&1 ');
 define('SAFE_FILE_LOCK', '/var/webconfig/tmp/safe.lock');
+
+define('ADD_NONE_ALWAYS', 'always');
+define('ADD_NONE_NEVER', 'never');
+define('ADD_NONE_IF_EMPTY', 'empty');
+
 /**
  * Authenticate, if not logged it's redirect at a login screen;
  *
@@ -91,6 +96,7 @@ function safe_redirect($url)
     <script type="text/javascript">
 window.location = "' . $url . '";
     </script>';
+    exit();
 }
 /**
  * Return the default web header
@@ -106,12 +112,14 @@ function safe_webheader($title = 'SAFe -', $summary = '', $large_icon = '', $sma
     // TODO: handle title.
     if (isset($GLOBALS['SAFE_WC_HELPER'])) {
         $html = SAFE_WC_HEADER($title);
+        /*
         $html.= <<<EOD
 <script type="text/javascript">
     $('#clearos_left_menu').accordion({
       autoHeight: false, active: 0 });
     </script>
 EOD;
+         */
         $html.= SAFE_WC_DIALOGINTRO($title, $summary, $large_icon, $small_icon);
         return $html;
     } else {
@@ -183,7 +191,7 @@ EOD;
 function safe_webtabletitleopen($title = '', $width = '100%', $id = '', $extra_class = '', $extra_style = '', $padding = '3px')
 {
     if (isset($GLOBALS['SAFE_WC_HELPER'])) {
-        return SAFE_WC_TABLEOPEN($title, $width, $id = '', $extra_class, $extra_style = '', $padding = '3px');
+        return SAFE_WC_TABLEOPEN($title, $width, $id, $extra_class, $extra_style = '', $padding = '3px');
     } else {
         return <<<EOD
     <div width='$width'>$title</div>
@@ -657,7 +665,7 @@ function safe_exec($command, $line_options = '', $arguments = '', &$return_var =
         $command.= implode(' ', $arguments) . ' ';
     }
     if ($escapeshell) $command = escapeshellcmd($command);
-    $comman = $command . SAFE_CMD_STDERR_REDIRECT;
+    $command = $command . SAFE_CMD_STDERR_REDIRECT;
     exec($command, $output, $return_var);
     // error_log ( "Safe_exec:" . $command ." ".implode(' ', $output));
     return $output;
@@ -698,7 +706,7 @@ function safe_var_dump($v)
 ?>
 <br />
 <pre
-  style="display: block; text-align: left; color: black; background-color: yellow; font-size: x-small;">
+  style="display: block; text-align: left; color: black; background-color: yellow; font-size: small;">
 
 <?php
     echo gettype($v) . ' called from: ';
@@ -713,17 +721,23 @@ function safe_var_dump($v)
 /**
  * Echo the debug backtrace
  */
-function safe_debug_info()
+function safe_debug_info($display=true)
 {
-    echo '<pre style="display: block; text-align: left; color: black; background-color: #D5D5D5; font-size: medium;">';
+    if($display){
+        echo '<pre style="display: block; text-align: left; color: black; background-color: #D5D5D5; font-size: medium;">';
+    }
     foreach (debug_backtrace() as $k => $v) {
         if ($k) {
             $l = safe_debug_info_line($v, $k);
-            echo htmlentities($l);
+            if($display){
+                echo htmlentities($l);
+            }
             error_log($l);
         }
     }
-    echo "</pre>";
+    if($display){
+        echo "</pre>";
+    }
 }
 /**
  * Create a string from a backtrace line.
@@ -997,6 +1011,7 @@ function Safe_format_xml($xml)
                 global $safe_error_buffer;
                 switch ($errno) {
                 case E_ERROR:
+                case E_USER_ERROR:
                 case E_CORE_ERROR:
                 case E_COMPILE_ERROR:
                 case E_WARNING:
@@ -1154,7 +1169,56 @@ function Safe_format_xml($xml)
         }
         return $port_used;
     }
-    
+    /**
+     * @brief safe_create_enum_list() add default enumeration optional value and label
+     *
+     *  @param[in out] $type - (string) Object type
+     *  @param[in out] $add_none (string) Add none entry in enumeration or not
+     *  ADD_NONE_ALWAYS => always add the none option
+     *  ADD_NONE_NEVER  => never add the none option
+     *  ADD_NONE_IF_EMPTY => only add the none option when the enum list is empty
+     *
+     *  @param[in out] $optional - (boolean) Is none optional
+     *  when true: Add __none__ in enumeration -> No value selected as part of optional selection,
+     *      this is allowed meaning user don't want to select one - no specific validation rule.
+     *  when false: Add __selectone__ to enumeration -> No value selected, this should turn into a rule
+     *      validating __selectone__ is not selected, meaning selection is MANDATORY
+     *
+     *  @param[in out] $none_label - (string) None enumeration label, default= '( None )', optional
+     *
+     *  Return an array of obj name list for enumeration
+     *
+     * @return
+     */
+    function safe_create_enum_list($data = array(), $add_none = ADD_NONE_ALWAYS, $optional = false, $none_label = null ) {
+        if($add_none == ADD_NONE_NEVER){
+            return $data;
+        }else{
+            $tmp_rs = array();
+            if($optional){
+                if($none_label){
+                    $tmp_rs['__none__'] = $none_label;
+                }else{
+                    $tmp_rs['__none__'] = '( None )';
+                }
+            }else{
+                if($none_label){
+                    $tmp_rs['__selectone__'] = $none_label;
+                }else{
+                    $tmp_rs['__selectone__'] = '( Please Select One )';
+                }
+            }
+            if($add_none == ADD_NONE_ALWAYS){
+                return array_merge($tmp_rs, $data);
+            }else{
+                if($data){
+                    return $data;
+                }else{
+                    return $tmp_rs;
+                }
+            }
+        }
+    }
     function safe_cidr_match($ipaddr, $cidr) {
         $ip = '';
         foreach (explode('.', $ipaddr) as $val) {
