@@ -19,11 +19,16 @@
 */
 require_once ('application/helpers/safe_helper.php');
 require_once ('safe_object_class.php');
+require_once ('system/libraries/Zip.php');
+require_once ('application/libraries/safe_upload_class.php');
+
 class Safe_module_class extends Safe_object_class
 {
     private $_node = null;
     private $_config = null;
     private $_modules = array();
+    private $_agg_obj_enum = array();
+    protected $_audit_parameters = array();
     public function __construct($software, $name, $type = "module")
     {
         parent::__construct($software->object_name() , $name);
@@ -45,17 +50,24 @@ class Safe_module_class extends Safe_object_class
     {
         return $this->_config;
     }
-    public function allow_rest(){
-        return false;
+
+    public function documentation()
+    {
+        $doc_dir = $this->node()->configuration_manager()->directory('doc', null, false);
+        $doc_file = $doc_dir.'/'.$this->name().'.txt';
+        $doc = array();
+        if($this->node()->os()->read_file($doc_file, $doc, true)){
+            return $doc;
+        }
+        return parent::documentation();
     }
-    public function set_config($config)
+    public function set_config($config, $controller_url=null)
     {
         unset($this->_config);
         if($config) {
             // Get data to expose parameters
             $data = $config->get_data(false);
-            // expose config as aggregate object
-            $this->register_aggregate_object( 'configuration',
+            $cfg_def = 
                 array(
                     'name' => 'Configuration',
                     'singleton' => true,
@@ -70,8 +82,12 @@ class Safe_module_class extends Safe_object_class
                         )
                     ),
                     'class' => $data,
-                )
-            );
+                );
+            if($controller_url){
+                $cfg_def['controller_url'] = $controller_url;
+            }
+            // expose config as aggregate object
+            $this->register_aggregate_object('configuration', $cfg_def);
         }
         $this->_config = $config;
         return true;
@@ -94,28 +110,6 @@ class Safe_module_class extends Safe_object_class
         }
         return false;
     }
-    /**
-     * @brief REST GET data
-     *         
-     * @return
-     */
-    public function rest_get_data()
-    {
-        $data = array();
-        // If config present, add it
-        if (isset($this->_config)) {
-            $data['configuration'] = $this->_config->rest_get_data();
-        }
-        // Ask parent to add its stuff
-        $rest_data = parent::rest_get_data();
-        if ($data) $rest_data = array_merge($rest_data, $data);
-        // Check for modules
-        foreach ($this->modules() as $module) {
-            $data = $module->rest_get_data();
-            if ($data) $rest_data[$module->name()] = $data;
-        }
-        return $rest_data;
-    }
     public function find_module_by_name($name)
     {
         if($name == $this->name()) {
@@ -229,6 +223,10 @@ class Safe_module_class extends Safe_object_class
         }
         return true;
     }
+    public function post_reload()
+    {
+        return true;
+    }
     public function register_module($module)
     {
         $this->_modules[$module->name() ] = $module;
@@ -250,6 +248,16 @@ class Safe_module_class extends Safe_object_class
         }
         return parent::configure();
     }
+    public function post_configure()
+    {
+        // Configure all modules
+        foreach ($this->_modules as $k => $v) {
+            if (true != $v->post_configure()){
+                return false;
+            }
+        }
+        return true;
+    }
     /**
      * @brief Return array of variables that need to be defined globally for
      *         application (key=>value)
@@ -305,6 +313,25 @@ class Safe_module_class extends Safe_object_class
     {
         return array();
     }
+    public function get_aggregate_object_method_params($type, $method)
+    {
+        $param = parent::get_aggregate_object_method_params($type, $method);
+        if( $param === false){
+            $definition_all = $this->get_data_settings();
+            $definition = $definition_all[$type];
+            if($definition['methods'][$method]){
+                $method_name = 'api_'.$method.'_'.$type.'_parameters';
+                if(method_exists($this, $method_name)) {
+                    $param =  call_user_func_array( array( $this, $method_name), array());
+                    return $param;
+                }
+            }
+            return false;
+        }else{
+            return $param;
+        }
+        
+    }
     /**
      * @brief Return count of specific data name
      *           
@@ -328,19 +355,31 @@ class Safe_module_class extends Safe_object_class
      *           
      * @return array( 'key' => 'file path' ) for each data entries
      */
+    private $_data_files_cache = array();
     public function get_data_files($data_name)
     {
         $data_cfg = $this->get_data_settings();
         if (isset($data_cfg[$data_name]) && true == $data_cfg[$data_name]['filesystem'] && isset($data_cfg[$data_name]['pattern'])) {
-            $data_files = array();
-            $files = $this->node()->os()->search_files($data_cfg[$data_name]['pattern'], $data_cfg[$data_name]['recursive']);
-            foreach ($files as $file) {
-                $data_files[$this->get_data_file_key($data_name, $file) ] = $file;
+            if(!$this->_data_files_cache[$data_name]){
+                $data_files = array();
+                $files = $this->node()->os()->search_files($data_cfg[$data_name]['pattern'], $data_cfg[$data_name]['recursive']);
+                foreach ($files as $file) {
+                    $data_files[$this->get_data_file_key($data_name, $file) ] = $file;
+                }
+                $this->_data_files_cache[$data_name] = $data_files;
             }
-            return $data_files;
+            return $this->_data_files_cache[$data_name];
         }
         throw new Exception('Module ' . $this->description() . ' get_data_files error for ' . $data_name);
     }
+    public function data_summary($data_name){
+        $data_files = $this->get_data_files($data_name);
+        $data_summary = array();
+        foreach ($data_files as $key => $file) {
+            $data_summary[] = array('Name' => $key);
+        }
+        return $data_summary;
+    }
     /**
      * @brief Retrieve data file key
      *           Default impletmentation if returning file name (basename)
@@ -384,7 +423,6 @@ class Safe_module_class extends Safe_object_class
             $os->chown_file($file, 'webconfig');
             // Read content
             if (isset($line_count)) {
-                $os = $this->node()->os();
                 $output = array();
                 $rc = $os->execute('sudo tail -n ' . $line_count . ' ' . $file, $output);
                 $content = implode('\n', $output);
@@ -465,6 +503,135 @@ class Safe_module_class extends Safe_object_class
         }
         throw new Exception('Module ' . $this->description() . ' delete_data error for ' . $data_name);
     }
+    /**
+     * @brief download all data
+     *
+     * @param[in out] $data_name
+     *
+     * @return
+     */
+    public function download_data($data_name, $data_key = NULL)
+    {
+        $data_cfg = $this->get_data_settings();
+        if (isset($data_cfg[$data_name]) && true == $data_cfg[$data_name]['filesystem'] && isset($data_cfg[$data_name]['pattern'])) {
+            $data_files = $this->get_data_files($data_name);
+            if (isset($data_key)) {
+                if (isset($data_files[$data_key])) {
+                    $data_files = array($data_key => $data_files[$data_key]);
+                }else{ 
+                    $data_files = array();
+                }
+            }
+            $os = $this->node()->os();
+            foreach ($data_files as $key => $file) {
+                $os->chown_file($file, 'webconfig');
+            }
+            if (count($data_files) > 1) {
+                // All of them, zip and dowload
+                ini_set('memory_limit', '300M');
+                // Ready to ZIIIIP....
+                $zip_obj =  new CI_Zip();
+                $name = php_uname('n') . '-' . $data_name . '-' . date('Y-m-d-H-i-s') . '.zip';
+                foreach ($data_files as $key => $file) {
+                    $rc = $zip_obj->read_file($file);
+                }
+                // Proceed to download
+                $rc = $zip_obj->download($name);
+            } else if (count($data_files)) {
+                // Download the file
+                if($data_key){
+                    safe_download($data_files[$data_key], NULL, $data_key);
+                }else{
+                    $data_key = key($data_files);
+                    safe_download($data_files[$data_key], NULL, $data_key);
+                }
+                
+            } 
+        }
+        header('HTTP/1.0 404 Not Found');
+        exit();
+    }
+    /**
+     * @brief upload data
+     *
+     * @param[in out] $data_name
+     *
+     * @return
+     */
+    public function upload_data($data_name, $data_key = NULL, &$output , $from = 'POST')
+    {
+        $data_cfg_setting = $this->get_data_settings();
+        if (isset($data_cfg_setting[$data_name]) && true == $data_cfg_setting[$data_name]['filesystem'] && isset($data_cfg_setting[$data_name]['pattern'])) {
+            $data_cfg = $data_cfg_setting[$data_name];
+            if($from == 'PUT'){
+                //convert put upload file to post upload file
+                $_POST['Upload'] = 'YES';
+                if (!isset($_SERVER['HTTP_UPLOAD_FILE_NAME'])) {
+                    $output['notif_error'][] = 'Fail to upload ' . $this->description() . ' ' . $data_cfg['description'];
+                    return false;
+                }else{
+                    $putdata = fopen("php://input", "r");
+                    $tmpfname = tempnam(sys_get_temp_dir(), "put_upload_");
+                    $fp = fopen($tmpfname, "w");
+                    while ($data = fread($putdata, 1024)){
+                        fwrite($fp, $data);
+                    }
+                    fclose($fp);
+                    fclose($putdata);
+                    $_FILES['archive']['name'] = $_SERVER['HTTP_UPLOAD_FILE_NAME'];
+                    $_FILES['archive']['type'] = 'application/octet-stream';
+                    $_FILES['archive']['tmp_name'] = $tmpfname;
+                    $_FILES['archive']['error'] = 0;
+                    $_FILES['archive']['size'] = filesize($tmpfname);
+                    $_FILES['archive']['from'] = 'PUT';
+                }
+            }
+            $upload_lib = new Safe_upload_class();
+            if ($upload_lib->is_uploading()) {
+                if($data_cfg['singleton']){
+                    //only allow one file
+                    $this->delete_data($data_name);
+                }
+                ini_set('memory_limit', '300M');
+                // path and ext are based data_cfg pattern
+                $ext = pathinfo($data_cfg['pattern']);
+                $upload_lib->set_upload_path($ext['dirname']);
+                if($data_cfg['file_ext']){
+                    $upload_lib->set_allowed_types($data_cfg['file_ext']);
+                }else{
+                    $upload_lib->set_allowed_types($ext['extension']);
+                }
+                $upload_lib->set_max_size(300 * 1024);
+                $rc = $upload_lib->upload();
+                $up_data = $upload_lib->get_data();
+                if ($rc) {
+                    if($from == 'PUT'){
+                        @unlink($tmpfname);
+                    }
+                    if($data_cfg['singleton']){
+                        //after upload ini
+                        $output_msg = array();
+                        if($this->initialize_data($data_name, $up_data['file_name'], $output_msg, $data_key)){
+                            return true;
+                        }else{
+                            $output['message'] = 'Fail to initialize '.$data_cfg['description'].'! ';
+                            if($output_msg){
+                                $output['reason'] = $output_msg;
+                            }
+                        }
+                    }else{
+                        return true;
+                    }
+                }else{
+                    $output['message'] = 'Fail to upload ' . $this->description() . ' ' . $data_cfg['description'] . '.';
+                    $output['reason'] = $upload_lib->get_errors();
+                }
+            }else{
+                $output['message'] = 'Fail to upload ' . $this->description() . ' ' . $data_cfg['description'];
+            }
+        }
+        return false;
+    }
     public function initialize_data($data_name, $data_key, &$output=null)
     {
         return false;
@@ -478,9 +645,21 @@ class Safe_module_class extends Safe_object_class
     {
         return false;
     }
-    public function can_install_data($data_name, $data_key, &$output=null)
+    /**
+     * @brief Check if update package can be install
+     *
+     * @param[in out] $reason
+     *
+     * @return true/false
+     */
+    public function can_install_data($info, &$reason)
     {
-        return false;
+        // Default is true
+        // Loop all sub-modules
+        foreach ($this->_modules as $k => $v) {
+            if (true != $v->can_install_data($info, $reason)) return false;
+        }
+        return true;
     }
     /**
      * @brief
@@ -502,11 +681,11 @@ class Safe_module_class extends Safe_object_class
      *           
      * @return true/false
      */
-    public function can_backup(&$reason)
+    public function can_backup($type, &$reason)
     {
         // Loop all sub-modules
         foreach ($this->_modules as $k => $v) {
-            if (true != $v->can_backup($reason)) return false;
+            if (true != $v->can_backup($type, $reason)) return false;
         }
         return true;
     }
@@ -517,12 +696,19 @@ class Safe_module_class extends Safe_object_class
      *           
      * @return true/false
      */
-    public function pre_backup(&$reason)
+    public function pre_backup($type, &$reason)
     {
         error_log('pre_backup - '.$this->name());
+        $script = $this->get_hook_script('pre', 'backup');
+        if($script) {
+            error_log('pre_backup - Invoking ' . $script . ' ' . $type);
+            $this->node()->execute('nohup '.$script . ' ' . $type, $retCode, true, true, $rc);
+            //db forcing re-opening db, table struct may be altered by this hooks
+            Safe_object_serializer_class::get_serializer(true);
+        }
         // Loop all sub-modules
         foreach ($this->_modules as $k => $v) {
-            if (true != $v->pre_backup($reason)) return false;
+            if (true != $v->pre_backup($type, $reason)) return false;
         }
         return true;
     }
@@ -533,12 +719,19 @@ class Safe_module_class extends Safe_object_class
      *           
      * @return true/false
      */
-    public function post_backup(&$reason)
+    public function post_backup($type, &$reason)
     {
         error_log('post_backup - '.$this->name());
+        $script = $this->get_hook_script('post', 'backup');
+        if($script) {
+            error_log('post_backup - Invoking ' . $script);
+            $this->node()->execute('nohup '.$script.' '.$type, $retCode, true, true, $rc);
+            //db forcing re-opening db, table struct may be altered by this hooks
+            Safe_object_serializer_class::get_serializer(true);
+        }
         // Loop all sub-modules
         foreach ($this->_modules as $k => $v) {
-            if (true != $v->post_backup($reason)) return false;
+            if (true != $v->post_backup($type, $reason)) return false;
         }
         return true;
     }
@@ -549,12 +742,12 @@ class Safe_module_class extends Safe_object_class
      *           
      * @return true/false
      */
-    public function can_restore(&$reason)
+    public function can_restore($info, &$reason)
     {
         // Default is true
         // Loop all sub-modules
         foreach ($this->_modules as $k => $v) {
-            if (true != $v->can_restore($reason)) return false;
+            if (true != $v->can_restore($info, $reason)) return false;
         }
         return true;
     }
@@ -565,12 +758,22 @@ class Safe_module_class extends Safe_object_class
      *           
      * @return true/false
      */
-    public function pre_restore(&$reason)
+    public function pre_restore($info, &$reason)
     {
         error_log('pre_restore - '.$this->name());
+        $script = $this->get_hook_script('pre', 'restore');
+        if($script) {
+            error_log('pre_restore - Invoking ' . $script . ' ' . $archive);
+            if(!$this->node()->execute('nohup '.$script . ' ' . $info['type'], $retCode, true, true, $rc)){
+                $reason = $retCode;
+                return false;
+            }
+            //db forcing re-opening db, table struct may be altered by this hooks
+            Safe_object_serializer_class::get_serializer(true);
+        }
         // Loop all sub-modules
         foreach ($this->_modules as $k => $v) {
-            if (true != $v->pre_restore($reason)) return false;
+            if (true != $v->pre_restore($info, $reason)) return false;
         }
         return true;
     }
@@ -581,9 +784,17 @@ class Safe_module_class extends Safe_object_class
      *           
      * @return true/false
      */
-    public function post_restore(&$reason)
+    public function post_restore($info, &$reason)
     {
         error_log('post_restore - '.$this->name());
+        // Check if product script exists
+        $script = $this->get_hook_script('post', 'restore');
+        if($script) {
+            error_log('post_restore - Invoking ' . $script);
+            $this->node()->execute('nohup '.$script. ' ' . $info['type'], $retCode, true, true, $rc);
+            //db forcing re-opening db, table struct may be altered by this hooks
+            Safe_object_serializer_class::get_serializer(true);
+        }
         // Make sure config is synched from DB
         if($this->config()){
             $this->config()->synch();
@@ -591,15 +802,26 @@ class Safe_module_class extends Safe_object_class
         $this->clear_cache();
         // Loop all sub-modules
         foreach ($this->_modules as $k => $v) {
-            if (true != $v->post_restore($reason)) return false;
+            if (true != $v->post_restore($info, $reason)) return false;
         }
         return true;
     }
+    protected function get_hook_script($step , $type)
+    {
+        $base = $this->node()->configuration_manager()->directory('base');
+        $script = $base.'/hooks/'.$this->name().'.' . $type .'.'.$step;
+        if(file_exists($script)) {
+            return $script;
+        }else{
+            return '';
+        }
+    }
     private function _modified_aggregate_object_list($obj, &$obj_list, $reloadable_only=true)
     {
         if ($agg_obj = $obj->has_aggregate_objects(true)) {
             // Loop around aggregate obj type
             foreach($agg_obj as $obj_type => $obj_type_name) {
+                if($obj_type_name['configurable'] === false) continue;
                 $objs = $obj->get_aggregate_objects($obj_type);
                 // loop around obj
                 foreach($objs as $k => $v) {
@@ -643,21 +865,27 @@ class Safe_module_class extends Safe_object_class
             // Check for config modified
             $obj_list[$this->name()]['configuration'] = $this->config();
         }
+        if(!$reloadable_only && $this->is_configuration_modified()) {
+            // Check for config modified
+            $obj_list[$this->name()]['configuration'] = $this->config();
+        }
         // Check aggregate objects
         foreach($this->has_aggregate_objects(true) as $k => $v ) {
             // Check if this object is dynamic or if any child (which can be 
             // dyanmic)
+            if($v['configurable'] === false) continue;
             if($v['dynamic'] || $v['has_child']) {
                 // Get those objects and check for modified ones
                 $objs = $this->get_aggregate_objects($k, true);
                 foreach($objs as $obj_name => $obj) {
-                    if($v['dynamic'] && $obj->status() != Safe_object_serializer_class::OBJ_STATUS_UP_TO_DATE) {
+                    if(($v['dynamic'] || !$reloadable_only)  && $obj->status() != Safe_object_serializer_class::OBJ_STATUS_UP_TO_DATE) {
                         $obj_list[$this->name()][$k][$obj_name] = $obj;
                     }
                     if($v['has_child'] && $obj->is_configuration_modified(true)) {
                         $child_objs_def = $obj->has_aggregate_objects(true);
                         foreach($child_objs_def as $child_obj_type => $child_obj_def) {
-                            if($child_obj_def['dynamic']) {
+                            if($child_obj_def['configurable'] === false) continue;
+                            if($child_obj_def['dynamic'] || !$reloadable_only) {
                                 $child_objs = $obj->get_aggregate_objects($child_obj_type, true);
                                 foreach($child_objs as $child_obj_name => $child_obj) {
                                     if($child_obj->is_modified()) {
@@ -714,6 +942,38 @@ class Safe_module_class extends Safe_object_class
         }
         return $conflicts;
     }
+    public function check_object_usage($src_module_name, $obj_type, $obj_name, $sub_typ=null, $sub_name=null)
+    {
+        $usages = array();
+        foreach($this->modules() as $mod) {
+            // Skip ourself :)
+            if($src_module_name == $mod->name()){
+                continue;
+            }
+            $sub_usages = array();
+            $sub_usages = $mod->check_object_usage($src_module_name, $obj_type, $obj_name, $sub_typ, $sub_name);
+            if($sub_usages){
+                $usages = array_merge($usages, $sub_usages);
+            }
+        }
+        return $usages;
+    }
+    public function check_object_modified($src_module_name, $obj_type, $obj_name, $sub_typ=null, $sub_name=null)
+    {
+        $modifies = array();
+        foreach($this->modules() as $mod) {
+            // Skip ourself :)
+            if($src_module_name == $mod->name()){
+                continue;
+            }
+            $sub_modifies = array();
+            $sub_modifies = $mod->check_object_modified($src_module_name, $obj_type, $obj_name, $sub_typ, $sub_name);
+            if($sub_modifies){
+                $modifies = array_merge($modifies, $sub_modifies);
+            }
+        }
+        return $modifies;
+    }
     /**
      * @brief Module checklist
      * 
@@ -729,5 +989,56 @@ class Safe_module_class extends Safe_object_class
         }
         return count($list);
     }
+    public function set_audit_params($params)
+    {
+        $this->_audit_parameters = $params;
+    }
+    public function get_audit_params()
+    {
+        return $this->_audit_parameters;
+    }
+    public function get_audit_points(&$audit_points)
+    {
+        // Create our auditpoints
+        foreach($this->_audit_parameters as $name => $point){
+            if($point['type']){
+                $audit_class = 'Safe_audit_point_'.$point['type'];
+                if(class_exists($audit_class)){
+                    $obj = new $audit_class($this, $point['name']);
+                    $obj->set_parameters($point);
+                    $audit_points[] = $obj;
+                }else{
+                    error_log("WARNING - ".$this->name().": Cannot instantiate audit point <${audit_class}>");
+                }
+            }
+        }
+        // Loop around modules
+        foreach ($this->modules() as $module) {
+            $module->get_audit_points($audit_points);
+        }
+    }
+    public function get_object_enum($type)
+    {
+        // Cached obj enumeration
+        if(!$this->_agg_obj_enum[$type]){
+            $this->_agg_obj_enum[$type] = array();
+            $objects = $this->get_aggregate_objects($type);
+            if($objects != null){
+                foreach($objects as $object){
+                    $this->_agg_obj_enum[$type][$object->name()] = $object->name();
+                }
+            }
+        }
+        return $this->_agg_obj_enum[$type];
+    }
+    public function object_enum($type, $add_none = ADD_NONE_ALWAYS, $optional = false, $none_label = null )
+    {
+        $object_enum = $this->get_object_enum($type);
+        return safe_create_enum_list($object_enum, $add_none, $optional, $none_label);
+    }
+    public function process_name()
+    {
+        return $this->name();
+    }
 }
 /* End of file safe_module_class.php */
