@@ -38,6 +38,8 @@ class Safe_table_class
     public $table; // CI_Table
     var $heading = array();
     var $data = array();
+    var $data_formatter = array();
+    var $apply_data_formatter = true;
     var $primary_key = array();
     var $line_button = array(); //Usage $this->line_button[] = array(array('url=>'test','name'=>'test'),array('url'=>'test2','name'=>'test2'));
     var $footer_button = array(); //Usage $this->footer_button = array(array ('url' => 'ntg_statistics/drop_sessions', 'name'=> 'Drop All Sessions'));
@@ -68,12 +70,12 @@ class Safe_table_class
         'td_stopped' => '<td class="stopped">',
         'td_started' => '<td class="started">',
         'td_error' => '<td class="error">',
-        'row_start' => '<tr>',
+        'row_start' => '<tr class="odd">',
         'row_end' => '</tr>',
         // depending of the table a nowrap attribute may be interesting.
         'cell_start' => '<td style="text-align:left; white-space:nowrap;">',
         'cell_end' => '</td>',
-        'row_alt_start' => '<tr class="mytablealt">',
+        'row_alt_start' => '<tr class="even">',
         'row_alt_end' => '</tr>',
         'cell_alt_start' => '<td style="text-align:left;">',
         'cell_alt_end' => '</td>',
@@ -390,36 +392,146 @@ class Safe_table_class
         }
         return $this->data;
     }
+    public function generate_data()
+    {
+        $data = array(
+            'heading' => $this->heading,
+            'row' => array(
+                'data' => $this->data,
+                'key' => $this->primary_key,
+                'button' => $this->line_button,
+            ),
+            'footer' => array(
+                'button' => $this->footer_button,
+                'data' => $this->footer,
+            ),
+        );
+
+        return $data;
+    }
+    protected function _data_formatter_none($data)
+    {
+        return $data;
+
+    }
+    protected function _data_formatter_pre($data, $recurse=false)
+    {
+        $html = '';
+        if(!$recurse){
+            $html = '<pre>';
+        }
+        if(is_array($data)){
+            foreach($data as $d){
+                if(is_array($d)){
+                    $html .= $this->_data_formatter_pre($d, true);
+                }else{
+                    $html .= $data.PHP_EOL;
+                }
+            }
+        }else{
+            $html .= htmlentities($data);
+        }
+
+        if(!$recurse){
+            $html .= '</pre>';
+        }
+        return $html;
+    }
+    protected function _data_formatter_list($data, $style='none', $recurse=false)
+    {
+        $padding = ($recurse)?'':' padding: 0px; margin: 0px;';
+        $html = '<ul style="list-style-type:'.$style.'; '.$padding.';">';
+        foreach($data as $d){
+            if(is_array($d)){
+                $html .= $this->_data_formatter_list($d, $style, true);
+            }else{
+                $html .= '<li>';
+                $html .= htmlentities($d);
+                $html .='</li>';
+            }
+        }
+        $html .= '</ul>';
+        return $html;
+    }
     public function generate()
     {
         $html = "";
         $array_html = "";
         $skip = false;
-        if (empty($this->template['table_open']) && !empty($this->template['my_table_open'])) $html.= $this->template['my_table_open'];
+        if (empty($this->template['table_open']) && !empty($this->template['my_table_open'])){
+            $html.= $this->template['my_table_open'];
+        }
         $heading = $this->generate_heading();
-        if (!empty($heading)) $this->table->set_heading($heading);
+        if (!empty($heading)){
+            $this->table->set_heading($heading);
+        }
         $i = 0;
         if ($this->data) {
             //$this->line_button is an array of arrays which contain buttons per line.
             foreach ($this->data as $l) {
+                // XSS filter on all data but buttons
+                if($this->apply_data_formatter){
+                    $filtered_l = array();
+                    $j = 0;
+                    foreach($l as $v){
+                        // Use data + style... format ?
+                        $tmp_v = null;
+                        if($v['data']){
+                            $tmp_v = $v;
+                            $v = $tmp_v['data'];
+                        }
+                        $filtered_v = '';
+                        // Check is data_style associated with this entry?
+                        if($this->data_formatter[$j]){
+                            $_formatter = '_data_formatter_'.$this->data_formatter[$j];
+                            if(method_exists($this, $_formatter)){
+                                $filtered_v = call_user_func(array($this, $_formatter),$v);
+                            }else{
+                                $filtered_v = $this->_data_formatter_pre($v);
+                            }
+                        }elseif(is_array($v)){
+                            $filtered_v = $this->_data_formatter_list($v);
+                        }else{
+                            $filtered_v = htmlentities($v);
+                        }
+                        $j++;
+                        // Put back in data + style ... format
+                        if($tmp_v){
+                            $tmp_v['data'] = $filtered_v;
+                            $filtered_v = $tmp_v;
+                        }
+                        $filtered_l[] = $filtered_v;
+                    }
+                    $l = $filtered_l;
+                }
+                // proceed with buttons
                 if (!empty($this->line_button) && count($this->line_button[$i]) > 0) {
                     $html0 = "";
                     $html0.= "<div id='buttons" . $i . "' style='float:right;'>";
                     foreach ($this->line_button[$i] as $button) {
-                        if (isset($button['extra_arguements']) && isset($button['extra_class'])) $html0.= $this->button($button, (!empty($this->primary_key[$i]) ? $this->primary_key[$i] : '') , $button['extra_arguements'], $button['extra_class']);
-                        elseif (isset($button['extra_arguements'])) $html0.= $this->button($button, (!empty($this->primary_key[$i]) ? $this->primary_key[$i] : '') , $button['extra_arguements'], '');
-                        elseif (isset($button['extra_class'])) $html0.= $this->button($button, (!empty($this->primary_key[$i]) ? $this->primary_key[$i] : '') , '', $button['extra_class']);
-                        else $html0.= $this->button($button, (!empty($this->primary_key[$i]) ? $this->primary_key[$i] : ''));
+                        if (isset($button['extra_arguements']) && isset($button['extra_class'])){
+                            $html0.= $this->button($button, (!empty($this->primary_key[$i]) ? $this->primary_key[$i] : '') , $button['extra_arguements'], $button['extra_class']);
+                        }elseif (isset($button['extra_arguements'])){
+                            $html0.= $this->button($button, (!empty($this->primary_key[$i]) ? $this->primary_key[$i] : '') , $button['extra_arguements'], '');
+                        }elseif (isset($button['extra_class'])){
+                            $html0.= $this->button($button, (!empty($this->primary_key[$i]) ? $this->primary_key[$i] : '') , '', $button['extra_class']);
+                        }else{
+                            $html0.= $this->button($button, (!empty($this->primary_key[$i]) ? $this->primary_key[$i] : ''));
+                        }
                     }
                     $html0.= "</div>";
                     $l[] = $html0;
                 }
                 if (is_array($l) && count($l[0]) > 2) {
                     $array_html = '<pre style="white-space: pre-wrap;white-space: -moz-pre-wrap;white-space: -pre-wrap;white-space: -o-pre-wrap;word-wrap: break-word;">';
-                    foreach ($l[0] as $value) $array_html.= $value . "<br />";
+                    foreach ($l[0] as $value){
+                        $array_html.= $value . "<br />";
+                    }
                     $array_html.= '</pre>';
                     $this->table->add_row($array_html);
-                } else $this->table->add_row($l);
+                } else{
+                    $this->table->add_row($l);
+                }
                 $i++;
             }
         }else{
