@@ -61,6 +61,7 @@ class Sng_control_panel extends Safe_CI_Controller
         // Checklist table
         $data['table_title'] = 'Configuration Checklist';
         $table = new Safe_table_class();
+        $table->apply_data_formatter = false;
         foreach($this->_checklist as $item){
             $module = $this->_the_app->find_module_by_name($item['module']);
             if($module){
@@ -187,8 +188,9 @@ class Sng_control_panel extends Safe_CI_Controller
                     if ($service->allow_user_ctl()) {
                         $line_button = array(
                             array(
-                                'url' => 'sng_control_panel/stop',
-                                'name' => 'Stop'
+                                'url' => 'sng_control_panel/stop_confirm',
+                                'name' => 'Stop',
+                                'type' => 'confirm'    
                             ) ,
                         );
                     }
@@ -200,9 +202,10 @@ class Sng_control_panel extends Safe_CI_Controller
                     if ($service->allow_user_ctl()) {
                         $line_button = array(
                             array(
-                                'url' => 'sng_control_panel/start',
+                                'url' => 'sng_control_panel/start_confirm',
                                 'name' => 'Start',
-                                'label' => ''
+                                'label' => '',
+                                'type' => 'confirm'
                             ) ,
                         );
                     }
@@ -216,9 +219,10 @@ class Sng_control_panel extends Safe_CI_Controller
                     $status_class = 'error';
                     $line_button = array(
                         array(
-                            'url' => 'sng_control_panel/reset',
+                            'url' => 'sng_control_panel/reset_confirm',
                             'name' => 'Reset',
-                            'label' => ''
+                            'label' => '',
+                            'type' => 'confirm'
                         )
                     );
                     $error[] = $service->description();
@@ -235,9 +239,9 @@ class Sng_control_panel extends Safe_CI_Controller
                         'data' => $status,
                         'class' => $status_class
                     ) ,
-                    $stats['cputime'],
-                    $stats['%cpu'],
-                    $stats['%mem']
+                    $stats['cpu']['time'],
+                    $stats['cpu']['percent'],
+                    $stats['mem']['percent']
                 ));
                 //Creates the primary key of each line to pass when the form is parsed
                 //$table->primary_key = array(1,2);
@@ -249,8 +253,10 @@ class Sng_control_panel extends Safe_CI_Controller
                     'name' => 'Refresh'
                 )
             );
-            $tmp_data['table'] = $table->generate();
-            $data_tables[] = $tmp_data;
+            if($table->data){
+                $tmp_data['table'] = $table->generate();
+                $data_tables[] = $tmp_data;
+            }
         }
         //Calls function that checks for notification messages
         $sng_control_lib->check_errors($unrecognized, $error, $data['notif_error'], $table, $data);
@@ -261,6 +267,35 @@ class Sng_control_panel extends Safe_CI_Controller
         foreach ($data_tables as $data_table) $this->load->view("sng_control_panel", $data_table);
         $this->load->view("safe_footer", $data);
     }
+    public function start_confirm($service_name)
+    {
+        $service = $this->_the_product->local_node()->software()->service($service_name);
+        $data['page_title'] = 'Service ' . $service->description() ;
+        if ($service->can_start($service_result)) {
+            $data['message'] = 'Confirm to start  service : <b>' . $service->description() . '</b>?';
+            $data['message_type'] = 'notice';
+            $data['result'] = true;
+            $data['action_url'] = base_url($this->router->fetch_class()). '/start/'. $service_name;
+            $data['setting']['title'] = 'Starting ' .$service->description();
+            $data['setting']['sec'] = '60';
+            $data['setting']['msg'] = 'Ooops ! Server time out...';
+            $data['setting']['label'] = 'Back';
+            $data['setting']['url'] = base_url('sng_control_panel');
+            echo json_encode($data);
+        }else{
+            $data['message'] = 'Start ' .$service_name .' error.';
+            $reason = array();
+            foreach ($service_result as $failure_reason) {
+                $reason[] = $failure_reason['msg'];
+            }
+            if($reason != null){
+                $data['message'] =  $data['message'] .'<br/>' .implode('<br/>',$reason);
+            }
+            $data['message_type'] = 'alert';
+            $data['result'] = false;
+            echo json_encode($data);
+        }
+    }
     //Function that runs when starting a process
     public function start($service_name)
     {
@@ -274,43 +309,50 @@ class Sng_control_panel extends Safe_CI_Controller
         //$the_product = Safe_get_product();
         $service = $this->_the_product->local_node()->software()->service($service_name);
         // Set the basic data
-        $data['page_title'] = 'Control Panel';
-        $data['page_summary'] = 'This page allows controlling Application hardware and software.';
-        $data['page_large_icon'] = '/images/icons/32x32/icon-register.png';
-        $data['page_small_icon'] = '/images/icons/16x16/icon-register.png';
-        $data['errors'] = array();
         if ($service->can_start($service_result)) {
             $result = $service->start($output);
             $mode = $service->set_startup(Safe_service_class::STARTUP_AUTOMATIC);
             // Redirect to main page on success
-            if (!$result && $mode) safe_redirect(base_url('sng_control_panel'));
-            //Declare Table classes to be used
-            $table = new Safe_table_class();
-            //Prepare the Control Panel Table Title
-            $data['table_title'] = "Starting " . $service_name;
-            $table->line_button = null;
-            $table->data = $output;
-            $table->footer_button = array(
-                array(
-                    'url' => base_url('sng_control_panel') ,
-                    'name' => 'Back'
-                )
-            );
-            $data['table'] = $table->generate();
-            $start_service = true;
+            if (!$result && $mode){
+                $data['result'] = true;
+                $data['redirect_url'] =  base_url('sng_control_panel');
+                echo json_encode($data);
+                exit();
+            }
+            $data['notif_warn'] = $output;
         } else {
             foreach ($service_result as $failure_reason) {
                 if (isset($failure_reason['url'])) {
                     $data['notif_error'][] = safe_form_with_submit_button($failure_reason['msg'], $failure_reason['url'], $failure_reason['label']);
-                } else $data['notif_warn'][] = $failure_reason['msg'];
+                } else {
+                    $data['notif_warn'][] = $failure_reason['msg'];
+                }
             }
-            $start_service = false;
         }
-        $this->load->view("safe_header", $data);
-        $this->load->view("safe_error");
-        $this->load->view("safe_notifications", $data);
-        if ($start_service) $this->load->view("sng_control_panel_start", $data);
-        $this->load->view("safe_footer", $data);
+        //return error message
+        $data['message'] = 'Start ' .$service_name .' error.';
+        if($data['notif_warn']){
+            $data['message'] =  $data['message'] .'<br/>' .implode('<br/>',$data['notif_warn']);
+            unset($data['notif_warn']);
+        }
+        $data['message_type'] = 'alert';
+        $data['result'] = false;
+        echo json_encode($data);
+    }
+    public function reset_confirm($service_name)
+    {
+        $service = $this->_the_product->local_node()->software()->service($service_name);
+        $data['page_title'] = 'Reset Service ' . $service->description() ;
+        $data['message'] = 'Confirm to reset  service : <b>' . $service->description() . '</b>?';
+        $data['message_type'] = 'notice';
+        $data['result'] = true;
+        $data['action_url'] = base_url($this->router->fetch_class()). '/reset/'. $service_name;
+        $data['setting']['title'] = 'Resetting '.$service->description();
+        $data['setting']['sec'] = '60';
+        $data['setting']['msg'] = 'Ooops ! Server time out...';
+        $data['setting']['lable'] = 'Back';
+        $data['setting']['url'] = base_url('sng_control_panel');
+        echo json_encode($data);
     }
     public function reset($service_name)
     {
@@ -322,31 +364,16 @@ class Sng_control_panel extends Safe_CI_Controller
         $service = $this->_the_product->local_node()->software()->service($service_name);
         $service->reset($retCode);
         // Redirect to main page on success
-        if ($retCode == 0) safe_redirect(base_url('sng_control_panel'));
-        else {
-            //Declare Table classes to be used
-            $table = new Safe_table_class();
-            //Prepare the Control Panel Table Title
-            $data['table_title'] = "Resetting " . $service_name;
-            $table->line_button = null;
-            $table->data[] = 'Unable to Reset Service ' . $service_name;
-            $table->footer_button = array(
-                array(
-                    'url' => base_url('sng_control_panel') ,
-                    'name' => 'Back'
-                )
-            );
-            $data['table'] = $table->generate();
-            // Set the basic data
-            $data['page_title'] = 'Control Panel';
-            $data['page_summary'] = 'This page allows controlling NSC hardware and software.';
-            $data['page_large_icon'] = '/images/icons/32x32/icon-register.png';
-            $data['page_small_icon'] = '/images/icons/16x16/icon-register.png';
-            $data['errors'] = array();
-            $this->load->view("safe_header", $data);
-            $this->load->view("safe_error");
-            $this->load->view("sng_control_panel_start", $data);
-            $this->load->view("safe_footer", $data);
+        if ($retCode == 0) {
+            $data['result'] = true;
+            $data['redirect_url'] =  base_url('sng_control_panel');
+            echo json_encode($data);
+            exit();
+        }else {
+            $data['message'] = 'Reset ' .$service_name .' error.';
+            $data['message_type'] = 'alert';
+            $data['result'] = false;
+            echo json_encode($data);
         }
     }
     //Function that runs when restarting a process
@@ -377,7 +404,7 @@ class Sng_control_panel extends Safe_CI_Controller
         $data['table'] = $table->generate();
         // Set the basic data
         $data['page_title'] = 'Control Panel';
-        $data['page_summary'] = 'This page allows controlling NTG hardware and software.';
+        $data['page_summary'] = 'This page allows controlling Application hardware and software.';
         $data['page_large_icon'] = '/images/icons/32x32/icon-register.png';
         $data['page_small_icon'] = '/images/icons/16x16/icon-register.png';
         $data['errors'] = array();
@@ -387,6 +414,21 @@ class Sng_control_panel extends Safe_CI_Controller
         $this->load->view("sng_control_panel_start", $data);
         $this->load->view("safe_footer", $data);
     }
+    public function stop_confirm($service_name)
+    {
+        $service = $this->_the_product->local_node()->software()->service($service_name);
+        $data['page_title'] = 'Service ' . $service->description() ;
+        $data['message'] = 'Confirm to stop  service : <b>' . $service->description() . '</b>?';
+        $data['message_type'] = 'notice';
+        $data['result'] = true;
+        $data['action_url'] = base_url($this->router->fetch_class()). '/stop/'. $service_name;
+        $data['setting']['title'] = 'Stopping '.$service->description() ;
+        $data['setting']['sec'] = '60';
+        $data['setting']['msg'] = 'Ooops ! Server time out...';
+        $data['setting']['lable'] = 'Back';
+        $data['setting']['url'] = base_url('sng_control_panel');
+        echo json_encode($data);
+    }
     //Function that runs when stopping a process
     public function stop($service_name)
     {
@@ -401,31 +443,17 @@ class Sng_control_panel extends Safe_CI_Controller
         $result = $service->stop($output);
         $mode = $service->set_startup(Safe_service_class::STARTUP_MANUAL);
         // Redirect to main page on success
-        if (!$result && $mode) safe_redirect(base_url('sng_control_panel'));
-        //Declare Table classes to be used
-        $table = new Safe_table_class();
-        //Prepare the Control Panel Table Title
-        $data['table_title'] = "Stopping " . $service_name;
-        $table->line_button = null;
-        $table->data = $output;
-        $table->footer_button = array(
-            array(
-                'url' => base_url('sng_control_panel') ,
-                'name' => 'Back'
-            )
-        );
-        $data['table'] = $table->generate();
-        // Set the basic data
-        $data['page_title'] = 'Control Panel';
-        $data['page_summary'] = 'This page allows controlling Application hardware and software.';
-        $data['page_large_icon'] = '/images/icons/32x32/icon-register.png';
-        $data['page_small_icon'] = '/images/icons/16x16/icon-register.png';
-        $data['errors'] = array();
-        $this->load->view("safe_header", $data);
-        $this->load->view("safe_error");
-        //$this->output->append_output( $service_name );
-        $this->load->view("sng_control_panel_start", $data);
-        $this->load->view("safe_footer", $data);
+        if (!$result && $mode) {
+            $data['result'] = true;
+            $data['redirect_url'] =  base_url('sng_control_panel');
+            echo json_encode($data);
+            exit();
+        }else{
+            $data['message'] = 'Stop ' .$service_name .' error.';
+            $data['message_type'] = 'alert';
+            $data['result'] = false;
+            echo json_encode($data);
+        }
     }
     //Changes the service mode
     public function set_mode($mode, $service_name)
